==> mutual_credit-5.0.1.tar.gz: mutual_credit/modules/limits/src/Form/WalletLimitOverride.php <==
<?php

namespace Drupal\mcapi_limits\Form;

use Drupal\mcapi_limits\WalletLimiter;
use Drupal\mcapi\Entity\Wallet;
use Drupal\mcapi\Entity\Currency;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
Use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Database\Connection;

/**
 * Form constructor for setting a wallet's limits.
 * @todo move all this onto the wallet manage form
 */
class WalletLimitOverride extends FormBase {

  private $database;
  private $walletLimiter;

  /**
   *
   * @param CurrentRouteMatch $routeMatch
   * @param Database $database
   * @param WalletLimiter $wallet_limiter
   */
  public function __construct(CurrentRouteMatch $routeMatch, Connection $database, WalletLimiter $wallet_limiter) {
    if ($wid = $routeMatch->getParameter('mcapi_wallet')) {
      $this->wallet = Wallet::load($wid);
    }
    $this->database = $database;
    $this->walletLimiter = $wallet_limiter->setwallet($this->wallet);
  }

  /**
   * Inject the services.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('database'),
      $container->get('mcapi_limits.wallet_limiter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'wallet_limits_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // This is tricky. We need to know all the currencies that could go in the
    // wallet. To do that we have to know all the currencies in the all the
    // active exchanges the wallets parent is in.
    $wallet = $this->wallet;
    $submit = FALSE;

    $overrides = $this->walletLimiter->overrides();
    foreach (Currency::loadMultiple() as $currency) {
      $config = $currency->getThirdPartySettings('mcapi_limits');
      if (!$config || $config['plugin'] == 'none') {
        continue;
      }
      $defaults = $this->walletLimiter->defaults($currency);
      $limits = array_filter($defaults);
      $desc = [];
      if (array_key_exists('min', $limits)) {
        $desc[] = t('Min: %worth', ['%worth' => $currency->format($limits['min'])]);
      }
      if (array_key_exists('max', $limits)) {
        $desc[] = t('Max: %worth', ['%worth' => $currency->format($limits['max'])]);
      }
      if ($config['override']) {
        $curr_id = $currency->id();
        // For now the per-wallet override allows admin to declare absolute min
        // and max per user. The next thing would be for the override to support
        // different plugins and peruser settings. This should be in the plugin.
        $form[$curr_id] = [
          '#title' => $currency->label(),
          '#description' => t("Leave blank to use the currency's default limits"),
          '#type' => 'details',
          '#open' => TRUE,
          '#tree' => TRUE,
          'min' => [
            '#title' => $this->t('Min') .' -',
            '#type' => 'worth_form',
            '#weight' => 0,
            '#default_value' => '',
            '#placeholder' => $defaults['min'] ? -$defaults['min'] : 0,
            '#allowed_curr_ids' => [$curr_id],
            '#config' => TRUE,
            '#minus' => TRUE,
          ],
          'max' => [
            '#title' => $this->t('Max'),
            '#type' => 'worth_form',
            '#weight' => 1,
            '#default_value' => '',
            '#placeholder' => $defaults['max'],
            '#allowed_curr_ids' => [$curr_id],
            '#config' => TRUE,
          ],
        ];
        if (isset($overrides[$curr_id])) {
          $vals = $overrides[$curr_id];
          if (isset($vals['min'])) {
            $form[$curr_id]['min']['#default_value'] = $vals['min']['value'];
          }
          if (isset($vals['max'])) {
            $form[$curr_id]['max']['#default_value'] = $vals['max']['value'];
          }
        }
        $submit = TRUE;
      }
    }
    if ($submit) {
      $form['help'] = [
        '#markup' => t("Leave fields blank to use the currencies' own settings") . '<br />',
        '#weight' => -1,
      ];
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => t('Save'),
        '#weight' => 10,
      ];
    }
    else {
      $form['empty']['#markup'] = t('This wallet has no currencies with overrideable balance limits');
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();
    foreach ($form_state->getValues() as $curr_id => $limits) {
      if ($limits['min']['value'] > 0) {
        $form_state->setErrorByName("$curr_id][min", $this->t('Min must be less than or equal to zero'));
      }
      if ($limits['max']['value'] < 0) {
        $form_state->setErrorByName("$curr_id][max", $this->t('Max must be greater than or equal to zero'));
      }
      if ($limits['min']['value'] == 0 && $limits['max']['value'] == 0) {
        $form_state->setErrorByName("$curr_id", $this->t('Min and max cannot both be zero'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();
    $wid = $this->wallet->id();
    // Clear db and rewrite.
    // Can't use upsert coz table has 2 field key
    $this->database
      ->delete('mcapi_wallets_limits')
      ->condition('wid', $wid)
      ->execute();
    $q = $this->database
      ->insert('mcapi_wallets_limits')
      ->fields(['wid', 'curr_id', 'max', 'value', 'editor', 'date']);
    $values = $form_state->getValues();
    // Rearrange the values so they are easier to save currency by currency.
    foreach ($values as $curr_id => $minmax) {
      foreach ($minmax as $limit => $worth) {
        if (!isset($worth['value'])) {
          continue;
        }
        $row = [
          'wid' => $wid,
          'curr_id' => $curr_id,
          'max' => (int) ($limit == 'max'),
          'value' => $worth['value'],
          'editor' => $this->currentUser()->id(),
          'date' => REQUEST_TIME,
        ];
        $q->values($row);
      }
    }
    if (isset($row)) {
      $q->execute();
    }
    else {
      $this->messenger()->addStatus(t('No limits were overridden'));
    }

    // No need to clear the wallet I think.
    $form_state->setRedirect(
      'entity.mcapi_wallet.canonical',
      ['mcapi_wallet' => $this->wallet->id()]
    );
  }

}

==> mutual_credit-5.0.8.tar.gz: mutual_credit/modules/limits/src/Form/WalletLimitOverride.php <==
<?php

namespace Drupal\mcapi_limits\Form;

use Drupal\mcapi_limits\WalletLimiter;
use Drupal\mcapi\Entity\Wallet;
use Drupal\mcapi\Entity\Currency;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
Use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Database\Connection;

/**
 * Form constructor for setting a wallet's limits.
 * @todo move all this onto the wallet manage form
 */
class WalletLimitOverride extends FormBase {

  private $database;
  private $walletLimiter;

  /**
   *
   * @param CurrentRouteMatch $routeMatch
   * @param Database $database
   * @param WalletLimiter $wallet_limiter
   */
  public function __construct(CurrentRouteMatch $routeMatch, Connection $database, WalletLimiter $wallet_limiter) {
    if ($wallet = $routeMatch->getParameter('mcapi_wallet')) {
      $this->wallet = $wallet;
    }
    $this->database = $database;
    $this->walletLimiter = $wallet_limiter->setwallet($this->wallet);
  }

  /**
   * Inject the services.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('database'),
      $container->get('mcapi_limits.wallet_limiter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'wallet_limits_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // This is tricky. We need to know all the currencies that could go in the
    // wallet. To do that we have to know all the currencies in the all the
    // active exchanges the wallets parent is in.
    $wallet = $this->wallet;
    $submit = FALSE;

    $overrides = $this->walletLimiter->overrides();
    foreach (Currency::loadMultiple() as $currency) {
      $config = $currency->getThirdPartySettings('mcapi_limits');
      if (!$config || $config['plugin'] == 'none') {
        continue;
      }
      $defaults = $this->walletLimiter->defaults($currency);
      $limits = array_filter($defaults);
      $desc = [];
      if (array_key_exists('min', $limits)) {
        $desc[] = t('Min: %worth', ['%worth' => $currency->format($limits['min'])]);
      }
      if (array_key_exists('max', $limits)) {
        $desc[] = t('Max: %worth', ['%worth' => $currency->format($limits['max'])]);
      }
      if ($config['override']) {
        $curr_id = $currency->id();
        // For now the per-wallet override allows admin to declare absolute min
        // and max per user. The next thing would be for the override to support
        // different plugins and peruser settings. This should be in the plugin.
        $form[$curr_id] = [
          '#title' => $currency->label(),
          '#description' => t("Leave blank to use the currency's default limits"),
          '#type' => 'details',
          '#open' => TRUE,
          '#tree' => TRUE,
          'min' => [
            '#title' => $this->t('Min') .' -',
            '#type' => 'worth_form',
            '#weight' => 0,
            '#default_value' => '',
            '#placeholder' => $defaults['min'] ? -$defaults['min'] : 0,
            '#allowed_curr_ids' => [$curr_id],
            '#config' => TRUE,
            '#minus' => TRUE,
          ],
          'max' => [
            '#title' => $this->t('Max'),
            '#type' => 'worth_form',
            '#weight' => 1,
            '#default_value' => '',
            '#placeholder' => $defaults['max'],
            '#allowed_curr_ids' => [$curr_id],
            '#config' => TRUE,
          ],
        ];
        if (isset($overrides[$curr_id])) {
          $vals = $overrides[$curr_id];
          if (isset($vals['min'])) {
            $form[$curr_id]['min']['#default_value'] = $vals['min']['value'];
          }
          if (isset($vals['max'])) {
            $form[$curr_id]['max']['#default_value'] = $vals['max']['value'];
          }
        }
        $submit = TRUE;
      }
    }
    if ($submit) {
      $form['help'] = [
        '#markup' => t("Leave fields blank to use the currencies' own settings") . '<br />',
        '#weight' => -1,
      ];
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => t('Save'),
        '#weight' => 10,
      ];
    }
    else {
      $form['empty']['#markup'] = t('This wallet has no currencies with overrideable balance limits');
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();
    foreach ($form_state->getValues() as $curr_id => $limits) {
      if ($limits['min']['value'] > 0) {
        $form_state->setErrorByName("$curr_id][min", $this->t('Min must be less than or equal to zero'));
      }
      if ($limits['max']['value'] < 0) {
        $form_state->setErrorByName("$curr_id][max", $this->t('Max must be greater than or equal to zero'));
      }
      if ($limits['min']['value'] == 0 && $limits['max']['value'] == 0) {
        $form_state->setErrorByName("$curr_id", $this->t('Min and max cannot both be zero'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();
    $wid = $this->wallet->id();
    // Clear db and rewrite.
    // Can't use upsert coz table has 2 field key
    $this->database
      ->delete('mcapi_wallets_limits')
      ->condition('wid', $wid)
      ->execute();
    $q = $this->database
      ->insert('mcapi_wallets_limits')
      ->fields(['wid', 'curr_id', 'max', 'value', 'editor', 'date']);
    $values = $form_state->getValues();
    // Rearrange the values so they are easier to save currency by currency.
    foreach ($values as $curr_id => $minmax) {
      foreach ($minmax as $limit => $worth) {
        if (!isset($worth['value'])) {
          continue;
        }
        $row = [
          'wid' => $wid,
          'curr_id' => $curr_id,
          'max' => (int) ($limit == 'max'),
          'value' => $worth['value'],
          'editor' => $this->currentUser()->id(),
          'date' => REQUEST_TIME,
        ];
        $q->values($row);
      }
    }
    if (isset($row)) {
      $q->execute();
    }
    else {
      $this->messenger()->addStatus(t('No limits were overridden'));
    }

    // No need to clear the wallet I think.
    $form_state->setRedirect(
      'entity.mcapi_wallet.canonical',
      ['mcapi_wallet' => $this->wallet->id()]
    );
  }

}
